'''
doc:
Subpackage for the TRXASprefitpack documentation

[deprecated]
This subpackage will be removed in version 0.5

:copyright: 2021 by pistack (Junho Lee).
:license: LGPL3.
'''

from .info import __info__

__all__ = ['__info__']
